@@ -1,4 +1,4 @@
-/**
+/*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * Common XML Doc utility functions
@@ -13,7 +13,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <errno.h>
 
 #include <libxml/tree.h>
 #include <libxml/valid.h>
@@ -27,6 +26,33 @@
 #include <xmlsec/base64.h>
 #include <xmlsec/errors.h>
 
+/**
+ * xmlSecFindSibling:
+ * @cur:                the pointer to XML node.
+ * @name:               the name.
+ * @ns:                 the namespace href (may be NULL).
+ *
+ * Searches @cur and the next siblings of the @cur node having given name and
+ * namespace href.
+ *
+ * Returns: the pointer to the found node or NULL if an error occurs or
+ * node is not found.
+ */
+xmlNodePtr
+xmlSecFindSibling(const xmlNodePtr cur, const xmlChar *name, const xmlChar *ns) {
+    xmlNodePtr tmp;
+    xmlSecAssert2(name != NULL, NULL);
+
+    for(tmp = cur; tmp != NULL; tmp = tmp->next) {
+        if(tmp->type == XML_ELEMENT_NODE) {
+            if(xmlSecCheckNodeName(tmp, name, ns)) {
+                return(tmp);
+            }
+        }
+    }
+    return(NULL);
+}
+
 /**
  * xmlSecFindChild:
  * @parent:             the pointer to XML node.
@@ -41,21 +67,10 @@
  */
 xmlNodePtr
 xmlSecFindChild(const xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
-    xmlNodePtr cur;
-
     xmlSecAssert2(parent != NULL, NULL);
     xmlSecAssert2(name != NULL, NULL);
 
-    cur = parent->children;
-    while(cur != NULL) {
-        if(cur->type == XML_ELEMENT_NODE) {
-            if(xmlSecCheckNodeName(cur, name, ns)) {
-                return(cur);
-            }
-        }
-        cur = cur->next;
-    }
-    return(NULL);
+    return(xmlSecFindSibling(parent->children, name, ns));
 }
 
 /**
@@ -186,11 +201,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
         /* TODO: add indents */
         text = xmlNewText(xmlSecStringCR);
         if(text == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlNewText",
-                        XMLSEC_ERRORS_R_XML_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+            xmlSecXmlError("xmlNewText", NULL);
             return(NULL);
         }
         xmlAddChild(parent, text);
@@ -198,11 +209,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 
     cur = xmlNewChild(parent, NULL, name, NULL);
     if(cur == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewChild",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewChild", NULL);
         return(NULL);
     }
 
@@ -214,6 +221,10 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
         nsPtr = xmlSearchNsByHref(cur->doc, cur, ns);
         if((nsPtr == NULL) || (xmlSearchNs(cur->doc, cur, nsPtr->prefix) != nsPtr)) {
             nsPtr = xmlNewNs(cur, ns, NULL);
+            if(nsPtr == NULL) {
+                xmlSecXmlError("xmlNewNs", NULL);
+                return(NULL);
+            }
         }
         xmlSetNs(cur, nsPtr);
     }
@@ -221,11 +232,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
     /* TODO: add indents */
     text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewText",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewText", NULL);
         return(NULL);
     }
     xmlAddChild(parent, text);
@@ -253,11 +260,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
         /* TODO: add indents */
         text = xmlNewText(xmlSecStringCR);
         if(text == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlNewText",
-                        XMLSEC_ERRORS_R_XML_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+            xmlSecXmlError("xmlNewText", NULL);
             return(NULL);
         }
         xmlAddChild(parent, text);
@@ -268,11 +271,7 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
     /* TODO: add indents */
     text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewText",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewText", NULL);
         return(NULL);
     }
     xmlAddChild(parent, text);
@@ -280,6 +279,52 @@ xmlSecAddChildNode(xmlNodePtr parent, xmlNodePtr child) {
     return(child);
 }
 
+/**
+ * xmlSecEnsureEmptyChild:
+ * @parent:             the pointer to XML node.
+ * @name:               the name.
+ * @ns:                 the namespace href (may be NULL).
+ *
+ * Searches a direct child of the @parent node having given name and
+ * namespace href. If not found then element node with given name / namespace
+ * is added.
+ *
+ * Returns: the pointer to the found or created node; or NULL if an error occurs.
+ */
+xmlNodePtr
+xmlSecEnsureEmptyChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
+    xmlNodePtr cur = NULL;
+    xmlNodePtr tmp;
+
+    xmlSecAssert2(parent != NULL, NULL);
+    xmlSecAssert2(name != NULL, NULL);
+
+    /* try to find an empty node first */
+    tmp = xmlSecFindNode(parent, name, ns);
+    while(tmp != NULL) {
+        cur = tmp;
+        if(xmlSecIsEmptyNode(cur) == 1) {
+            return(cur);
+        }
+        tmp = xmlSecFindSibling(cur->next, name, ns);
+    }
+
+    /* if not found then either add next or add at the end */
+    if(cur == NULL) {
+        cur = xmlSecAddChild(parent, name, ns);
+    } else if((cur->next != NULL) && (cur->next->type == XML_TEXT_NODE)) {
+        cur = xmlSecAddNextSibling(cur->next, name, ns);
+    } else {
+        cur = xmlSecAddNextSibling(cur, name, ns);
+    }
+    if(cur == NULL) {
+        xmlSecInternalError2("xmlSecAddChild or xmlSecAddNextSibling", NULL,
+                             "node=%s", xmlSecErrorsSafeString(name));
+        return(NULL);
+    }
+    return(cur);
+}
+
 /**
  * xmlSecAddNextSibling
  * @node:               the pointer to an XML node.
@@ -300,11 +345,7 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 
     cur = xmlNewNode(NULL, name);
     if(cur == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewNode",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewNode", NULL);
         return(NULL);
     }
     xmlAddNextSibling(node, cur);
@@ -324,11 +365,7 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     /* TODO: add indents */
     text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewText",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewText", NULL);
         return(NULL);
     }
     xmlAddNextSibling(node, text);
@@ -356,11 +393,7 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 
     cur = xmlNewNode(NULL, name);
     if(cur == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewNode",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewNode", NULL);
         return(NULL);
     }
     xmlAddPrevSibling(node, cur);
@@ -380,11 +413,7 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     /* TODO: add indents */
     text = xmlNewText(xmlSecStringCR);
     if(text == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewText",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewText", NULL);
         return(NULL);
     }
     xmlAddPrevSibling(node, text);
@@ -452,11 +481,7 @@ xmlSecReplaceNodeAndReturn(xmlNodePtr node, xmlNodePtr newNode, xmlNodePtr* repl
 
     oldNode = xmlReplaceNode(node, newNode);
     if(oldNode == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlReplaceNode",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlReplaceNode", NULL);
         return(-1);
     }
 
@@ -564,17 +589,15 @@ int
 xmlSecReplaceNodeBufferAndReturn(xmlNodePtr node, const xmlSecByte *buffer, xmlSecSize size, xmlNodePtr *replaced) {
     xmlNodePtr results = NULL;
     xmlNodePtr next = NULL;
+    int ret;
 
     xmlSecAssert2(node != NULL, -1);
     xmlSecAssert2(node->parent != NULL, -1);
 
     /* parse buffer in the context of node's parent */
-    if(xmlParseInNodeContext(node->parent, (const char*)buffer, size, XML_PARSE_NODICT, &results) != XML_ERR_OK) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlParseInNodeContext",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "Failed to parse content");
+    ret = xmlParseInNodeContext(node->parent, (const char*)buffer, size, XML_PARSE_NODICT, &results);
+    if(ret != XML_ERR_OK) {
+        xmlSecXmlError("xmlParseInNodeContext", NULL);
         return(-1);
     }
 
@@ -614,24 +637,17 @@ xmlSecNodeEncodeAndSetContent(xmlNodePtr node, const xmlChar * buffer) {
     xmlSecAssert2(node->doc != NULL, -1);
 
     if(buffer != NULL) {
-            xmlChar * tmp;
-
+        xmlChar * tmp;
         tmp = xmlEncodeSpecialChars(node->doc, buffer);
         if (tmp == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlEncodeSpecialChars",
-                        XMLSEC_ERRORS_R_XML_FAILED,
-                        "Failed to encode special characters");
+            xmlSecXmlError("xmlEncodeSpecialChars", NULL);
             return(-1);
         }
-
         xmlNodeSetContent(node, tmp);
         xmlFree(tmp);
     } else {
         xmlNodeSetContent(node, NULL);
     }
-
     return(0);
 }
 
@@ -666,12 +682,7 @@ xmlSecAddIDs(xmlDocPtr doc, xmlNodePtr cur, const xmlChar** ids) {
                         if(tmp == NULL) {
                             xmlAddID(NULL, doc, name, attr);
                         } else if(tmp != attr) {
-                            xmlSecError(XMLSEC_ERRORS_HERE,
-                                        NULL,
-                                        NULL,
-                                        XMLSEC_ERRORS_R_INVALID_DATA,
-                                        "id=%s already defined",
-                                        xmlSecErrorsSafeString(name));
+                            xmlSecInvalidStringDataError("id", name, "unique id (id already defined)", NULL);
                         }
                         xmlFree(name);
                     }
@@ -699,6 +710,8 @@ xmlSecAddIDs(xmlDocPtr doc, xmlNodePtr cur, const xmlChar** ids) {
  * @prefix:                     the prefix to add to the generated ID (can be NULL).
  * @len:                        the length of ID.
  *
+ * Deprecated and will be removed in the future releases, do not use.
+ *
  * Generates a unique ID in the format <@prefix>base64-encoded(@len random bytes)
  * and puts it in the attribute @attrName.
  *
@@ -716,22 +729,14 @@ xmlSecGenerateAndAddID(xmlNodePtr node, const xmlChar* attrName, const xmlChar*
     for(count = 0; count < 5; count++) {
         id = xmlSecGenerateID(prefix, len);
         if(id == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecGenerateID",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+            xmlSecInternalError("xmlSecGenerateID", NULL);
             return(-1);
         }
 
         if((node->doc == NULL) || (xmlGetID(node->doc, id) == NULL)) {
             /* this is a unique ID in the document and we can use it */
             if(xmlSetProp(node, attrName, id) == NULL) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "xmlSetProp",
-                            XMLSEC_ERRORS_R_XML_FAILED,
-                            XMLSEC_ERRORS_NO_MESSAGE);
+                xmlSecXmlError("xmlSetProp", NULL);
                 xmlFree(id);
                 return(-1);
             }
@@ -750,6 +755,8 @@ xmlSecGenerateAndAddID(xmlNodePtr node, const xmlChar* attrName, const xmlChar*
  * @prefix:                     the prefix to add to the generated ID (can be NULL).
  * @len:                        the length of ID.
  *
+ * Deprecated and will be removed in the future releases, do not use.
+ *
  * Generates a unique ID in the format <@prefix>base64-encoded(@len random bytes).
  * The caller is responsible for freeing returned string using @xmlFree function.
  *
@@ -770,11 +777,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 
     ret = xmlSecBufferInitialize(&buffer, binLen + 1);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBufferInitialize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBufferInitialize", NULL);
         return(NULL);
     }
     xmlSecAssert2(xmlSecBufferGetData(&buffer) != NULL, NULL);
@@ -782,11 +785,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
 
     ret = xmlSecBufferSetSize(&buffer, binLen);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBufferSetSize",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBufferSetSize", NULL);
         xmlSecBufferFinalize(&buffer);
         return(NULL);
     }
@@ -800,11 +799,7 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
     /* base64 encode random bytes */
     res = xmlSecBase64Encode(xmlSecBufferGetData(&buffer), xmlSecBufferGetSize(&buffer), 0);
     if((res == NULL) || (xmlStrlen(res) == 0)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecBase64Encode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecInternalError("xmlSecBase64Encode", NULL);
         xmlSecBufferFinalize(&buffer);
         return(NULL);
     }
@@ -830,16 +825,19 @@ xmlSecGenerateID(const xmlChar* prefix, xmlSecSize len) {
         tmpLen = xmlStrlen(prefix) + xmlStrlen(res) + 1;
         tmp = xmlMalloc(tmpLen + 1);
         if(tmp == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlMalloc",
-                        XMLSEC_ERRORS_R_MALLOC_FAILED,
-                        XMLSEC_ERRORS_NO_MESSAGE);
+            xmlSecMallocError(tmpLen + 1, NULL);
             xmlFree(res);
             return(NULL);
         }
 
-        xmlSecStrPrintf(tmp, tmpLen, BAD_CAST "%s%s", prefix, res);
+        ret = xmlStrPrintf(tmp, tmpLen, "%s%s", prefix, res);
+        if(ret < 0) {
+            xmlSecXmlError("xmlStrPrintf", NULL);
+            xmlFree(res);
+            xmlFree(tmp);
+            return(NULL);
+        }
+
         xmlFree(res);
         res = tmp;
     } else {
@@ -874,22 +872,15 @@ xmlSecCreateTree(const xmlChar* rootNodeName, const xmlChar* rootNodeNs) {
     /* create doc */
     doc = xmlNewDoc(BAD_CAST "1.0");
     if(doc == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewDoc",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    XMLSEC_ERRORS_NO_MESSAGE);
+        xmlSecXmlError("xmlNewDoc", NULL);
         return(NULL);
     }
 
     /* create root node */
     root = xmlNewDocNode(doc, NULL, rootNodeName, NULL);
     if(root == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewDocNode",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=Keys");
+        xmlSecXmlError2("xmlNewDocNode", NULL,
+                        "node=%s", rootNodeName);
         xmlFreeDoc(doc);
         return(NULL);
     }
@@ -898,12 +889,8 @@ xmlSecCreateTree(const xmlChar* rootNodeName, const xmlChar* rootNodeNs) {
     /* and set root node namespace */
     ns = xmlNewNs(root, rootNodeNs, NULL);
     if(ns == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNewNs",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "ns=%s",
-                    xmlSecErrorsSafeString(rootNodeNs));
+        xmlSecXmlError2("xmlNewNs", NULL,
+                        "ns=%s", xmlSecErrorsSafeString(rootNodeNs));
         xmlFreeDoc(doc);
         return(NULL);
     }
@@ -978,12 +965,8 @@ xmlSecPrintXmlString(FILE * fd, const xmlChar * str) {
         xmlChar * encoded_str = NULL;
         encoded_str = xmlEncodeSpecialChars(NULL, str);
         if(encoded_str == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlEncodeSpecialChars",
-                        XMLSEC_ERRORS_R_XML_FAILED,
-                        "string=%s",
-                        xmlSecErrorsSafeString(str));
+            xmlSecXmlError2("xmlEncodeSpecialChars", NULL,
+                            "string=%s", xmlSecErrorsSafeString(str));
             return(-1);
         }
 
@@ -994,12 +977,7 @@ xmlSecPrintXmlString(FILE * fd, const xmlChar * str) {
     }
 
     if(res < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "fprintf",
-                    XMLSEC_ERRORS_R_IO_FAILED,
-                    "res=%d,errno=%d",
-                    res, errno);
+        xmlSecIOError("fprintf", NULL, NULL);
         return(-1);
     }
     return(res);
@@ -1021,6 +999,7 @@ xmlChar*
 xmlSecGetQName(xmlNodePtr node, const xmlChar* href, const xmlChar* local) {
     xmlChar* qname;
     xmlNsPtr ns;
+    int ret;
 
     xmlSecAssert2(node != NULL, NULL);
     xmlSecAssert2(local != NULL, NULL);
@@ -1029,13 +1008,8 @@ xmlSecGetQName(xmlNodePtr node, const xmlChar* href, const xmlChar* local) {
      * it might cause collisions */
     ns = xmlSearchNsByHref(node->doc, node, href);
     if((ns == NULL) && (href != NULL)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSearchNsByHref",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=%s,href=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(href));
+        xmlSecXmlError2("xmlSearchNsByHref", NULL,
+                        "node=%s", xmlSecErrorsSafeString(node->name));
         return(NULL);
     }
 
@@ -1045,24 +1019,20 @@ xmlSecGetQName(xmlNodePtr node, const xmlChar* href, const xmlChar* local) {
         len = xmlStrlen(local) + xmlStrlen(ns->prefix) + 4;
         qname = xmlMalloc(len);
         if(qname == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlMalloc",
-                        XMLSEC_ERRORS_R_MALLOC_FAILED,
-                        "node=%s",
-                        xmlSecErrorsSafeString(node->name));
+            xmlSecMallocError(len, NULL);
+            return(NULL);
+        }
+
+        ret = xmlStrPrintf(qname, len, "%s:%s", ns->prefix, local);
+        if(ret < 0) {
+            xmlSecXmlError("xmlStrPrintf", NULL);
+            xmlFree(qname);
             return(NULL);
         }
-        xmlSecStrPrintf(qname, len, BAD_CAST "%s:%s", ns->prefix, local);
     } else {
         qname = xmlStrdup(local);
         if(qname == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlStrdup",
-                        XMLSEC_ERRORS_R_MALLOC_FAILED,
-                        "node=%s",
-                        xmlSecErrorsSafeString(node->name));
+            xmlSecStrdupError(local, NULL);
             return(NULL);
         }
     }
@@ -1164,13 +1134,7 @@ xmlSecQName2IntegerGetIntegerFromString(xmlSecQName2IntegerInfoConstPtr info,
     if(qnameLocalPart != NULL) {
         qnamePrefix = xmlStrndup(qname, qnameLocalPart - qname);
         if(qnamePrefix == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlStrndup",
-                        XMLSEC_ERRORS_R_MALLOC_FAILED,
-                        "node=%s,value=%s",
-                        xmlSecErrorsSafeString(node->name),
-                        xmlSecErrorsSafeString(qname));
+            xmlSecStrdupError(qname, NULL);
             return(-1);
         }
         qnameLocalPart++;
@@ -1182,13 +1146,8 @@ xmlSecQName2IntegerGetIntegerFromString(xmlSecQName2IntegerInfoConstPtr info,
     /* search namespace href */
     ns = xmlSearchNs(node->doc, node, qnamePrefix);
     if((ns == NULL) && (qnamePrefix != NULL)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSearchNs",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=%s,qnamePrefix=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(qnamePrefix));
+        xmlSecXmlError2("xmlSearchNs", NULL,
+                        "node=%s", xmlSecErrorsSafeString(node->name));
         if(qnamePrefix != NULL) {
             xmlFree(qnamePrefix);
         }
@@ -1199,14 +1158,11 @@ xmlSecQName2IntegerGetIntegerFromString(xmlSecQName2IntegerInfoConstPtr info,
     /* and finally search for integer */
     ret = xmlSecQName2IntegerGetInteger(info, qnameHref, qnameLocalPart, intValue);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetInteger",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,qnameLocalPart=%s,qnameHref=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(qnameLocalPart),
-                    xmlSecErrorsSafeString(qnameHref));
+        xmlSecInternalError4("xmlSecQName2IntegerGetInteger", NULL,
+                             "node=%s,qnameLocalPart=%s,qnameHref=%s",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(qnameLocalPart),
+                             xmlSecErrorsSafeString(qnameHref));
         if(qnamePrefix != NULL) {
             xmlFree(qnamePrefix);
         }
@@ -1241,13 +1197,10 @@ xmlSecQName2IntegerGetStringFromInteger(xmlSecQName2IntegerInfoConstPtr info,
 
     qnameInfo = xmlSecQName2IntegerGetInfo(info, intValue);
     if(qnameInfo == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetInfo",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,intValue=%d",
-                    xmlSecErrorsSafeString(node->name),
-                    intValue);
+        xmlSecInternalError3("xmlSecQName2IntegerGetInfo", NULL,
+                             "node=%s,intValue=%d",
+                             xmlSecErrorsSafeString(node->name),
+                             intValue);
         return(NULL);
     }
 
@@ -1276,25 +1229,18 @@ xmlSecQName2IntegerNodeRead(xmlSecQName2IntegerInfoConstPtr info, xmlNodePtr nod
 
     content = xmlNodeGetContent(node);
     if(content == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlNodeGetContent",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=%s",
-                    xmlSecErrorsSafeString(node->name));
+        xmlSecXmlError2("xmlNodeGetContent", NULL,
+                        "node=%s", xmlSecErrorsSafeString(node->name));
         return(-1);
     }
     /* todo: trim content? */
 
     ret = xmlSecQName2IntegerGetIntegerFromString(info, node, content, intValue);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetIntegerFromString",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,value=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(content));
+        xmlSecInternalError3("xmlSecQName2IntegerGetIntegerFromString", NULL,
+                             "node=%s,value=%s",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(content));
         xmlFree(content);
         return(-1);
     }
@@ -1328,25 +1274,19 @@ xmlSecQName2IntegerNodeWrite(xmlSecQName2IntegerInfoConstPtr info, xmlNodePtr no
     /* find and build qname */
     qname = xmlSecQName2IntegerGetStringFromInteger(info, node, intValue);
     if(qname == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetStringFromInteger",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,intValue=%d",
-                    xmlSecErrorsSafeString(node->name),
-                    intValue);
+        xmlSecInternalError3("xmlSecQName2IntegerGetStringFromInteger", NULL,
+                             "node=%s,intValue=%d",
+                             xmlSecErrorsSafeString(node->name),
+                             intValue);
         return(-1);
     }
 
     cur = xmlSecAddChild(node, nodeName, nodeNs);
     if(cur == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecAddChild",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,intValue=%d",
-                    xmlSecErrorsSafeString(nodeName),
-                    intValue);
+        xmlSecInternalError3("xmlSecAddChild", NULL,
+                             "node=%s,intValue=%d",
+                             xmlSecErrorsSafeString(nodeName),
+                             intValue);
         xmlFree(qname);
         return(-1);
     }
@@ -1381,27 +1321,19 @@ xmlSecQName2IntegerAttributeRead(xmlSecQName2IntegerInfoConstPtr info, xmlNodePt
 
     attrValue = xmlGetProp(node, attrName);
     if(attrValue == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlGetProp",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=%s,attrValue=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(attrName));
+        xmlSecXmlError2("xmlGetProp", NULL,
+                        "node=%s", xmlSecErrorsSafeString(node->name));
         return(-1);
     }
     /* todo: trim value? */
 
     ret = xmlSecQName2IntegerGetIntegerFromString(info, node, attrValue, intValue);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetIntegerFromString",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,attrName=%s,attrValue=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(attrName),
-                    xmlSecErrorsSafeString(attrValue));
+        xmlSecInternalError4("xmlSecQName2IntegerGetIntegerFromString", NULL,
+                             "node=%s,attrName=%s,attrValue=%s",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(attrName),
+                             xmlSecErrorsSafeString(attrValue));
         xmlFree(attrValue);
         return(-1);
     }
@@ -1435,27 +1367,21 @@ xmlSecQName2IntegerAttributeWrite(xmlSecQName2IntegerInfoConstPtr info, xmlNodeP
     /* find and build qname */
     qname = xmlSecQName2IntegerGetStringFromInteger(info, node, intValue);
     if(qname == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2IntegerGetStringFromInteger",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,attrName=%s,intValue=%d",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(attrName),
-                    intValue);
+        xmlSecInternalError4("xmlSecQName2IntegerGetStringFromInteger", NULL,
+                             "node=%s,attrName=%s,intValue=%d",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(attrName),
+                             intValue);
         return(-1);
     }
 
     attr = xmlSetProp(node, attrName, qname);
     if(attr == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecAddChildNode",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,attrName=%s,intValue=%d",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(attrName),
-                    intValue);
+        xmlSecInternalError4("xmlSecAddChildNode", NULL,
+                             "node=%s,attrName=%s,intValue=%d",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(attrName),
+                             intValue);
         xmlFree(qname);
         return(-1);
     }
@@ -1612,13 +1538,7 @@ xmlSecQName2BitMaskGetBitMaskFromString(xmlSecQName2BitMaskInfoConstPtr info,
     if(qnameLocalPart != NULL) {
         qnamePrefix = xmlStrndup(qname, qnameLocalPart - qname);
         if(qnamePrefix == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlStrndup",
-                        XMLSEC_ERRORS_R_MALLOC_FAILED,
-                        "node=%s,value=%s",
-                        xmlSecErrorsSafeString(node->name),
-                        xmlSecErrorsSafeString(qname));
+            xmlSecStrdupError(qname, NULL);
             return(-1);
         }
         qnameLocalPart++;
@@ -1630,13 +1550,8 @@ xmlSecQName2BitMaskGetBitMaskFromString(xmlSecQName2BitMaskInfoConstPtr info,
     /* search namespace href */
     ns = xmlSearchNs(node->doc, node, qnamePrefix);
     if((ns == NULL) && (qnamePrefix != NULL)) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSearchNs",
-                    XMLSEC_ERRORS_R_XML_FAILED,
-                    "node=%s,qnamePrefix=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(qnamePrefix));
+        xmlSecXmlError2("xmlSearchNs", NULL,
+                        "node=%s", xmlSecErrorsSafeString(node->name));
         if(qnamePrefix != NULL) {
             xmlFree(qnamePrefix);
         }
@@ -1647,14 +1562,11 @@ xmlSecQName2BitMaskGetBitMaskFromString(xmlSecQName2BitMaskInfoConstPtr info,
     /* and finally search for integer */
     ret = xmlSecQName2BitMaskGetBitMask(info, qnameHref, qnameLocalPart, mask);
     if(ret < 0) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2BitMaskGetBitMask",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,qnameLocalPart=%s,qnameHref=%s",
-                    xmlSecErrorsSafeString(node->name),
-                    xmlSecErrorsSafeString(qnameLocalPart),
-                    xmlSecErrorsSafeString(qnameHref));
+        xmlSecInternalError4("xmlSecQName2BitMaskGetBitMask", NULL,
+                             "node=%s,qnameLocalPart=%s,qnameHref=%s",
+                             xmlSecErrorsSafeString(node->name),
+                             xmlSecErrorsSafeString(qnameLocalPart),
+                             xmlSecErrorsSafeString(qnameHref));
         if(qnamePrefix != NULL) {
             xmlFree(qnamePrefix);
         }
@@ -1689,13 +1601,10 @@ xmlSecQName2BitMaskGetStringFromBitMask(xmlSecQName2BitMaskInfoConstPtr info,
 
     qnameInfo = xmlSecQName2BitMaskGetInfo(info, mask);
     if(qnameInfo == NULL) {
-        xmlSecError(XMLSEC_ERRORS_HERE,
-                    NULL,
-                    "xmlSecQName2BitMaskGetInfo",
-                    XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                    "node=%s,mask=%d",
-                    xmlSecErrorsSafeString(node->name),
-                    mask);
+        xmlSecInternalError3("xmlSecQName2BitMaskGetInfo", NULL,
+                             "node=%s,mask=%d",
+                             xmlSecErrorsSafeString(node->name),
+                             mask);
         return(NULL);
     }
 
@@ -1736,23 +1645,15 @@ xmlSecQName2BitMaskNodesRead(xmlSecQName2BitMaskInfoConstPtr info, xmlNodePtr* n
     while((cur != NULL) && (xmlSecCheckNodeName(cur, nodeName, nodeNs))) {
         content = xmlNodeGetContent(cur);
         if(content == NULL) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlNodeGetContent",
-                        XMLSEC_ERRORS_R_XML_FAILED,
-                        "node=%s",
-                        xmlSecErrorsSafeString(cur->name));
+            xmlSecXmlError2("xmlNodeGetContent", NULL,
+                            "node=%s", xmlSecErrorsSafeString(cur->name));
             return(-1);
         }
 
         ret = xmlSecQName2BitMaskGetBitMaskFromString(info, cur, content, &tmp);
         if(ret < 0) {
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecQName2BitMaskGetBitMaskFromString",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "value=%s",
-                        xmlSecErrorsSafeString(content));
+            xmlSecInternalError2("xmlSecQName2BitMaskGetBitMaskFromString", NULL,
+                                 "value=%s", xmlSecErrorsSafeString(content));
             xmlFree(content);
             return(-1);
         }
@@ -1760,12 +1661,8 @@ xmlSecQName2BitMaskNodesRead(xmlSecQName2BitMaskInfoConstPtr info, xmlNodePtr* n
 
         if((stopOnUnknown != 0) && (tmp == 0)) {
             /* todo: better error */
-            xmlSecError(XMLSEC_ERRORS_HERE,
-                        NULL,
-                        "xmlSecQName2BitMaskGetBitMaskFromString",
-                        XMLSEC_ERRORS_R_XMLSEC_FAILED,
-                        "value=%s",
-                        xmlSecErrorsSafeString(content));
+            xmlSecInternalError2("xmlSecQName2BitMaskGetBitMaskFromString", NULL,
+                                 "value=%s", xmlSecErrorsSafeString(content));
             return(-1);
         }
 
@@ -1808,23 +1705,15 @@ xmlSecQName2BitMaskNodesWrite(xmlSecQName2BitMaskInfoConstPtr info, xmlNodePtr n
 
             qname = xmlSecGetQName(node, info[ii].qnameHref, info[ii].qnameLocalPart);
             if(qname == NULL) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "xmlSecGetQName",
-                            XMLSEC_ERRORS_R_XML_FAILED,
-                            "node=%s",
-                            xmlSecErrorsSafeString(nodeName));
+                xmlSecXmlError2("xmlSecGetQName", NULL,
+                                "node=%s", xmlSecErrorsSafeString(nodeName));
                 return(-1);
             }
 
             cur = xmlSecAddChild(node, nodeName, nodeNs);
             if(cur == NULL) {
-                xmlSecError(XMLSEC_ERRORS_HERE,
-                            NULL,
-                            "xmlSecAddChild",
-                            XMLSEC_ERRORS_R_XML_FAILED,
-                            "node=%s",
-                            xmlSecErrorsSafeString(nodeName));
+                xmlSecXmlError2("xmlSecAddChild", NULL,
+                                "node=%s", xmlSecErrorsSafeString(nodeName));
                 xmlFree(qname);
                 return(-1);
             }
